local _
local L
local prefix = "NomiTab"
local debug
local guild
local playerName
local date
local libGraph
local graph

local function output(msg, omit)
	print((not omit and prefix .. " - " or "") .. msg)
end

local function dbg(msg)
	if debug then
		output(msg, 1)
	end
end

local function printHelp(type)
	if type and L[type] then
		output(L[type])
	else
		local help = {
				"'graph': show graph of throughput",
				"'compare': compare gains since <x> days ago (if no data present, use closest data)",
				"'help': display this text"
				}
		output("Available commands:")

		for i = 1, #help do
			output(help[i], 1)
		end
	end
end

local function getCmd(msg)
	if msg then
		local a,b,c = strfind(msg, "(%S+)")
		if a then
			return c, strsub(msg, b+2)
		else	
			return ""
		end
	end
end

local function initDate()
	guild = GetGuildInfo("player")

	if not guild then
		return
	end

	if not NomiTab_Data[guild] then
		NomiTab_Data[guild] = {}
	end

	if not NomiTab_Data[guild][date] then
		NomiTab_Data[guild][date] = {
					["\124cffffffff\124Hitem:74653:0:0:0:0:0:0:0:90:0:0\124h[Steamed Crab Surprise]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74646:0:0:0:0:0:0:0:90:0:0\124h[Black Pepper Ribs and Shrimp]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74656:0:0:0:0:0:0:0:90:0:0\124h[Chun Tian Spring Rolls]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74648:0:0:0:0:0:0:0:90:0:0\124h[Sea Mist Rice Noodles]\124h\124r"] = 0,
					["\124cffffffff\124Hitem:74650:0:0:0:0:0:0:0:90:0:0\124h[Mogu Fish Stew]\124h\124r"] = 0
				}
	end
end

local function getDate(ofsDay)
	ofsDay = tonumber(ofsDay)

	if not ofsDay or ofsDay < 0 then
		ofsDay = 0
	end

	return tonumber(_G["date"]("%y%m%d", time() - 86400 * ofsDay))		-- 86400 sec/24h
end

local function getLastDate()
	local tar, tot

	for i = 0, 365 do
		tar = getDate(i)

		if NomiTab_Data[guild][tar] then
			tot = 0

			for item, count in pairs(NomiTab_Data[guild][tar]) do
				tot = tot + count
			end

			if tot ~= 0 then
				return tar
			end
		end
	end

	return getDate(0)
end

local function updateDate()
	local old = date
	date = getDate()

	if not date or not guild or not NomiTab_Data[guild] or not NomiTab_Data[guild][date] then
		initDate()
	end
end

local function init()
	playerName = UnitName("player")
	date = 0

	if not NomiTab_Data then
		NomiTab_Data = {}
	end

	if not NomiTab_Data["nomiTab"] then
		NomiTab_Data["nomiTab"] = 1
	end

	updateDate()

	L = NomiTab_Loc["enUS"]
	NomiTab_Loc = nil

	NomiTab:UnregisterEvent("ADDON_LOADED")

	-----------------------------------------------------
	-----------------------------------------------------

	local h, w = 256, 512

	libGraph = LibStub:GetLibrary("LibGraph-2.0")

	graphFrame = CreateFrame("NomiTabGraphFrame", UIParent)
	graphFrame:SetSize(h, w)
	graphFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	graphFrame:Hide()

	graph = libGraph:CreateGraphScatterPlot("NomiTabGraphFrameGraph", graphFrame, "TOPLEFT", "TOPLEFT", 0, 0, h, w)

	-----------------------------------------------------
	-----------------------------------------------------
end

local function compareData(dataA, dataB, last, store)
	local diff = {}
	local msg = {}
	local balance = 0

	msg[1] = "Results since " .. 
			((last == getDate(0) and "earlier today") or (last == getDate(1) and "yesterday") or ((last .. ""):sub(5,6) .. "/" .. (last .. ""):sub(3,4))) .. ":"

	for itemLink, count in pairs(dataB) do
		diff[itemLink] = count - dataA[itemLink]

		if diff[itemLink] ~= 0 then
			msg[#msg + 1] = itemLink .. " : " .. dataA[itemLink] .. "|cff" .. ((diff[itemLink] > 0 and "00ff00+") or (diff[itemLink] < 0 and "ff0000-")) .. math.abs(diff[itemLink]) .. "|r"
		end

		balance = balance + diff[itemLink]
	end

	if not msg[2] then
		msg[2] = "No change!"
	end

	output(msg[1])
	for i = 2, #msg do
		output(msg[i], 1)
	end

	if balance ~= 0 then
		output("|cffff0000The sum is not zero! Difference of |r" .. balance .. "|cffff0000.|r", 1)
	end

	if store then
		for itemLink, count in pairs(dataB) do
			NomiTab_Data[guild][date][itemLink] = dataB[itemLink]
		end
	end
end

local function readTab(tab)
	local data = {}

	updateDate()

	local texturePath, itemCount, locked, itemLink

	for slot = 1, 98 do
		texturePath, itemCount, locked = GetGuildBankItemInfo(tab, slot)
		itemLink = GetGuildBankItemLink(tab, slot)

		if itemLink then
			data[itemLink] = (data[itemLink] or 0) + itemCount
		end
	end

	local last = getLastDate()

	compareData(NomiTab_Data[guild][last], data, last, true)
end

local function command(msg)
	local cmd, subCmd = getCmd(msg)

	if cmd == "graph" then
		local last, number = getCmd(subCmd)

		last = tonumber(last)
		number = tonumber(number)

		if last then
			if number then
				graph:ResetData()

				graph:SetXAxis(0, 200)
				graph:LockXMin(true)
				graph:LockXMax(true)

				graph:SetYAxis(last, number)
				graph:LockYMin(true)
				graph:LockYMax(true)
			else
				
			end
		else
			printHelp("notlast")
		end
	elseif cmd == "compare" then
		local tar, number = getCmd(subCmd)
		local cmpDate

		tar = tonumber(tar)

		if tar then
			local oldArr

			tar = math.abs(tar)

			while true do
				cmpDate = getDate(tar)

				oldArr = NomiTab_Data[guild][cmpDate]

				if oldArr then
					break
				end

				tar = tar - 1

				if tar == 0 then
					return
				end
			end

print(cmpDate,date)
print(oldArr["\124cffffffff\124Hitem:74653:0:0:0:0:0:0:0:90:0:0\124h[Steamed Crab Surprise]\124h\124r"],NomiTab_Data[guild][date]["\124cffffffff\124Hitem:74653:0:0:0:0:0:0:0:90:0:0\124h[Steamed Crab Surprise]\124h\124r"])

			compareData(oldArr, NomiTab_Data[guild][date], cmpDate)
		end
	elseif cmd == "tab" then
		local tar, _ = getCmd(subCmd)

		tar = tonumber(tar)

		if tar and tar < 10 and tar > 0 then
			NomiTab_Data["nomiTab"] = tar
		end
	elseif cmd == "dumpData" then
		if subCmd == "all" then
			for date,arr in pairs(NomiTab_Data[guild]) do
				print("---"..date.."---")
				for k,v in pairs(arr) do
					print(k,v)
				end
			end
		else
			for k,v in pairs(NomiTab_Data[guild][date]) do
				print(k,v)
			end
		end
	elseif cmd == "test" then
		for date,arr in pairs(NomiTab_Data[guild]) do
			print("-- " .. date .. " --")
			for k,v in pairs(arr) do
				print(k,v)
			end
		end
	else
		printHelp()
	end
end

function NomiTab_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == "NomiTab" then
		init()
	elseif event == "GUILDBANKFRAME_OPENED" then
		if GetCurrentGuildBankTab() ~= NomiTab_Data["nomiTab"] then
			SetCurrentGuildBankTab(NomiTab_Data["nomiTab"])
		end

		QueryGuildBankTab(NomiTab_Data["nomiTab"])

		readTab(NomiTab_Data["nomiTab"])
	elseif event == "GUILDBANKBAGSLOTS_CHANGED" then
		
	end
end

function NomiTab_OnLoad()
	local regs = {
			"ADDON_LOADED",
			"GUILDBANKFRAME_OPENED",
			"GUILDBANKBAGSLOTS_CHANGED"
		}

	for i = 1, #regs do
		NomiTab:RegisterEvent(regs[i])
	end

	SLASH_NOMITAB1 = "/nt"
	SLASH_NOMITAB2 = "/nomi"
	SLASH_NOMITAB3 = "/nomitab"
	SlashCmdList["NOMITAB"] = command
end